DJI7 API
---
for the core digital joystick interface (one or more players, 2-4 buttons supported)

The name DJI7 is short for Digital Joystick Interface, and a nice small number to distinguish it
from other digital joystick interfaces we may make for the CMM2, without suggesting priority or that they need to be in order.



There were some limitations I hadn't intended in the original API as implemented. First I'll describe what I was aiming for:

Version 3
---

- Library is a .inc file, which may be included by name

**Initialisation**

- SUB Controllers.Init players

Initialises the controllers library. "players" is the number of players/controllers to request. "players" may be omitted, in which case the default of 1 player is used.

May only be called once by the program.

If you request more players than supported by the library, when you check the extra unsupported controller numbers they will show no buttons pressed or directions moved.

**Directions**
- FUNCTION Controllers.Up(player)
- FUNCTION Controllers.Down(player)
- FUNCTION Controllers.Left(player)
- FUNCTION Controllers.Right(player)

Returns nonzero if the player is moving the joystick in that direction (or non-joystick equivalent). Returns zero if the player is not moving the joystick in that direction, or "player" is more than the number of players.
Diagonal movement is indicated by two functions returning nonzero, e.g., moving diagonally up and left is indicated Controllers.Up() being non-zero and Controllers.Left() being non zero.
For some non-joystick controllers, the player may be able to 

Player numbers start with 1. So for 5 players, the players are numbered 1,2,3,4, and 5. When it is 0 or omitted, and the library was initialised with only 1 player, that player, player 1, is checked.

**Buttons**

The "player" argument has the same meaning as for the directions.

- FUNCTION Controllers.ButtonA(player)

Returns nonzero if the player is pressing the primary button. Returns zero if the player is not pressing the primary button, or if there is no button.

- FUNCTION Controllers.ButtonB(player)

Returns nonzero if the player is pressing the secondary button. Returns zero if the player is not pressing the secondary button, or if there is no secondary button.

Now one of the limitations:

Version 2 and earlier of the API
---
If you request more players than supported by the library, the result somewhat undefined: there may be an error, or different player number may all report the results from the same joystick, or it may behave as in version 3.

And the other limitation:

Version 1 of the API
---
Due to a bug in the original version, the optional player number "player" may need to be omitted for these functions. The functions then use the default player 1.
- FUNCTION Controllers.Up(player)
- FUNCTION Controllers.Down(player)
- FUNCTION Controllers.Left(player)
- FUNCTION Controllers.Right(player)

- FUNCTION Controllers.ButtonA(player)
- FUNCTION Controllers.ButtonB(player)

e.g. instead it is FUNCTION Controllers.ButtonB() with no player argument.

Extensions and API versions
---

As an extension, the library may also support
- Controllers.ButtonC(player)
- Controllers.ButtonD(player)

As another extension, the library may give the API version like this:
`CONST Controllers.API.DJI7.Version = 2`
for version 2, or modified appropriately for versions 1, 3, or later.

Other ways to check versions and extensions: you can initialise the library, and check whether API version 2 is supported by seeing if you get an error with this code or similar:

`LOCAL testplayer = Controllers.Up(player)`

(this may be useful if the library doesn't set the API version number)

and check whether the additional buttons are supported by seeing if you get an error with this code or similar:

`LOCAL testbuttonc = Controllers.ButtonC(player)`

Similarly, there should be some mechanism for checking other APIs provided by these or related libraries.
